#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <gconf/gconf-client.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>

#define FONT_KEY "/apps/metacity/general/titlebar_font"
#define FONT_ACTIVE_KEY "/apps/metacity/general/titlebar_uses_system_font"

typedef enum _settings
{
    S_WAOC=0,
    S_WAIC,
    S_WAOA,
    S_WAIA,
    S_WADL,
    S_WAUA,
    S_BAOC,
    S_BAFC,
    S_BAA,
    S_TAOC,
    S_TAFC,
    S_WIOC,
    S_WIIC,
    S_WIOA,
    S_WIIA,
    S_BIOC,
    S_BIFC,
    S_BIA,
    S_TIOC,
    S_TIFC,
    S_WHC,
    S_SC,
    S_SR,
    S_SA,
    S_SXO,
    S_SYO,
    S_TH,
    S_AAPLY,
    SETTINGS_COUNT
} settings;

GtkWidget * SW[SETTINGS_COUNT]; // all the settings
GtkWidget * ThemeSelector;
GtkListStore * ThemeList;
GtkCellRenderer * ThemeRenderer;
GtkTreeViewColumn * ThemeColumn;
GtkTreeSelection * ThemeSelect;
GtkWidget * Cheating;
GtkWidget * Cheating2;
GtkWidget * mainWindow;
GtkWidget * AutoApply;
GtkWidget * EntryBox;
GtkWidget * FontButton;

GConfClient * gconf;

gboolean changed=FALSE;

//we'll say theme files end in .cgwdtheme
#define THEMEENDING ".cgwdtheme" 
static void theme_list_append(gchar * value)
{
    GtkTreeIter iter;
    gtk_list_store_append(ThemeList,&iter);
    gtk_list_store_set(ThemeList,&iter,0,value,-1);
}
static void theme_scan_dir(gchar * dir,gboolean writable)
{
    GDir * d;
    d = g_dir_open(dir,0,NULL);
    if (d)
    {
        gchar * n;
        while((n = (gchar *) g_dir_read_name(d)))
        {
            if (strlen(n)>=strlen(THEMEENDING))
            {
                if (strcmp(n+(strlen(n)-strlen(THEMEENDING)),THEMEENDING)==0)
                {
                    //actually add it here
                    gchar * o;
                    if (writable)
                        o = g_strdup(n);
                    else
                        o = g_strdup_printf("* %s",n);
                    o[strlen(o)-strlen(THEMEENDING)]='\0';
                    theme_list_append(o);
                    g_free(o);
                }
            }
        }
        g_dir_close(d);
    }
}
static void write_font(GKeyFile * f)
{
    g_key_file_set_string(f,"theme","titlebar_font",
            gtk_font_button_get_font_name(GTK_FONT_BUTTON(FontButton)));
}
static void read_font(GKeyFile * f)
{
    gchar * c;
    if ((c = g_key_file_get_string(f,"theme","titlebar_font",NULL)))
    {
        gtk_font_button_set_font_name(GTK_FONT_BUTTON(FontButton),c);
        g_free(c);
    }
}
static void refresh_theme_list()
{
    gchar * path;
    gtk_list_store_clear(ThemeList);
    theme_scan_dir(DATA_DIR "/compiz/themes/",FALSE); 
    path = g_strdup_printf("%s/.compiz/themes/",g_get_home_dir());
    theme_scan_dir(path,TRUE);
    g_free(path);
}
static gboolean confirm_dialog(gchar * val, gchar * val2)
{
    GtkWidget * w;
    int ret;
    w = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_MESSAGE_QUESTION,
            GTK_BUTTONS_YES_NO,
            val,
            val2);
    ret=gtk_dialog_run(GTK_DIALOG(w));
    gtk_widget_destroy(w);
    return (ret == GTK_RESPONSE_YES);
}
static void info_dialog(gchar * val)
{
    GtkWidget * w;
    w = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_MESSAGE_INFO,
            GTK_BUTTONS_CLOSE,
            val);
    gtk_dialog_run(GTK_DIALOG(w));
    gtk_widget_destroy(w);
}
static void error_dialog(gchar * val)
{
    GtkWidget * w;
    w = gtk_message_dialog_new(GTK_WINDOW(mainWindow),
            GTK_DIALOG_DESTROY_WITH_PARENT,
            GTK_MESSAGE_ERROR,
            GTK_BUTTONS_CLOSE,
            val);
    gtk_dialog_run(GTK_DIALOG(w));
    gtk_widget_destroy(w);
}
#define CDR "/apps/compiz/plugins/decoration/allscreens/options/"
#define get_float(key,object) gtk_range_set_value(GTK_RANGE(SW[(object)]),\
        gconf_client_get_float(gconf,CDR key,NULL))
#define get_int(key,object) gtk_range_set_value(GTK_RANGE(SW[(object)]),\
        gconf_client_get_int(gconf,CDR key,NULL))
#define get_bool(key,object) gtk_toggle_button_set_active(\
        GTK_TOGGLE_BUTTON(SW[(object)]),gconf_client_get_bool(gconf,CDR key,NULL))
static void get_color_real(GtkWidget * w, char * key)
{
    GdkColor col;
    gchar * c = gconf_client_get_string(gconf,key,NULL);
    if (c)
    {
        gdk_color_parse(c,&col);
        gtk_color_button_set_color(GTK_COLOR_BUTTON(w),&col);
        g_free(c);
    }
}
#define get_color(key,object) get_color_real(SW[(object)],CDR key )
static void init_settings()
{
    gconf = gconf_client_get_default();

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply)))
        return; // revert is meaningless here.
    
    get_float("shadow_radius",S_SR);
    get_float("shadow_opacity",S_SA);
    get_int("shadow_offset_x",S_SXO);
    get_int("shadow_offset_y",S_SYO);
    get_float("active_outer_alpha",S_WAOA);
    get_float("inactive_outer_alpha",S_WIOA);
    get_float("active_inner_alpha",S_WAIA);
    get_float("inactive_inner_alpha",S_WIIA);
    get_float("active_button_alpha",S_BAA);
    get_float("inactive_button_alpha",S_BIA);
    get_bool("use_new_decor",S_TH);
    get_bool("use_active_colors",S_WAUA);
    get_bool("draw_separator_line",S_WADL);
    get_color("active_outer",S_WAOC);
    get_color("active_inner",S_WAIC);
    get_color("inactive_outer",S_WIOC);
    get_color("inactive_inner",S_WIIC);
    get_color("active_text",S_TAFC);
    get_color("active_text_halo",S_TAOC);
    get_color("inactive_text",S_TIFC);
    get_color("inactive_text_halo",S_TIOC);
    get_color("active_button",S_BAFC);
    get_color("active_button_halo",S_BAOC);
    get_color("inactive_button",S_BIFC);
    get_color("inactive_button_halo",S_BIOC);
    get_color("window_frame_halo",S_WHC);
    get_color("shadow_color",S_SC);
    //let's figure out the font.
    //FIXME - this is inelegant and possibly obnoxious
    gconf_client_set_bool(gconf,FONT_ACTIVE_KEY,FALSE,NULL);
    {
        gchar * c = gconf_client_get_string(gconf,FONT_KEY,NULL);
        if (c)
        {
            gtk_font_button_set_font_name(GTK_FONT_BUTTON(FontButton),c);
            g_free(c);
        }
    }
}
static void set_font()
{
    gconf_client_set_string(gconf,FONT_KEY,
            gtk_font_button_get_font_name(GTK_FONT_BUTTON(FontButton)),NULL);
}
static void cb_set_font(GtkWidget * w, gpointer p)
{
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply)))
        set_font();
    else
        changed=TRUE;
}
#define set_float(key,object) gconf_client_set_float(gconf,CDR key,\
        gtk_range_get_value(GTK_RANGE(SW[(object)])),NULL)
#define set_int(key,object) gconf_client_set_int(gconf,CDR key,\
        gtk_range_get_value(GTK_RANGE(SW[(object)])),NULL)
#define set_bool(key,object) gconf_client_set_bool(gconf,CDR key,\
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SW[(object)])),NULL)
static void set_color_real(GtkWidget * w, char * key)
{
    GdkColor col;
    gchar * c;
    gtk_color_button_get_color(GTK_COLOR_BUTTON(w),&col);
    c=g_strdup_printf("#%02x%02x%02x",
            col.red>>8,
            col.green>>8,
            col.blue>>8);
    gconf_client_set_string(gconf,key,c,NULL);
    g_free(c);
}
#define set_color(key,object) set_color_real(SW[(object)],CDR key )
static void apply_settings()
{
    gconf_client_set_bool(gconf,CDR "applying", TRUE, NULL);
    set_float("shadow_radius",S_SR);
    set_float("shadow_opacity",S_SA);
    set_int("shadow_offset_x",S_SXO);
    set_int("shadow_offset_y",S_SYO);
    set_float("active_outer_alpha",S_WAOA);
    set_float("inactive_outer_alpha",S_WIOA);
    set_float("active_inner_alpha",S_WAIA);
    set_float("inactive_inner_alpha",S_WIIA);
    set_float("active_button_alpha",S_BAA);
    set_float("inactive_button_alpha",S_BIA);
    set_bool("use_active_colors",S_WAUA);
    set_bool("draw_separator_line",S_WADL);
    set_color("active_outer",S_WAOC);
    set_color("active_inner",S_WAIC);
    set_color("inactive_outer",S_WIOC);
    set_color("inactive_inner",S_WIIC);
    set_color("active_text",S_TAFC);
    set_color("active_text_halo",S_TAOC);
    set_color("inactive_text",S_TIFC);
    set_color("inactive_text_halo",S_TIOC);
    set_color("active_button",S_BAFC);
    set_color("active_button_halo",S_BAOC);
    set_color("inactive_button",S_BIFC);
    set_color("inactive_button_halo",S_BIOC);
    set_color("window_frame_halo",S_WHC);
    set_color("shadow_color",S_SC);
    set_font();
    gconf_client_set_bool(gconf,CDR "applying", FALSE, NULL);
}
void read_color_real(GtkWidget * w, char * key, GKeyFile * file)
{
    GdkColor c;
    gchar * ch;
    if ((ch = g_key_file_get_string(file,"theme",key,NULL)))
    {
        gdk_color_parse(ch,&c);
        gtk_color_button_set_color(GTK_COLOR_BUTTON(w),&c);
        g_free(ch);
    }
}
void read_int_real(GtkWidget * w, char * key, GKeyFile * file)
{
    GError * e = NULL;
    int i = g_key_file_get_integer(file,"theme",key,&e);
    if (!e)
        gtk_range_set_value(GTK_RANGE(w),i);
}
void read_float_real(GtkWidget * w, char * key, GKeyFile * file)
{
    double d;
    gchar * ch;
    if ((ch = g_key_file_get_string(file,"theme",key,NULL)))
    {
        d = g_ascii_strtod(ch,NULL); // - atof doesn't work for no real reason
        gtk_range_set_value(GTK_RANGE(w),d);
        g_free(ch);
    }
}
void read_bool_real(GtkWidget * w, char * key, GKeyFile * file)
{
    GError * e = NULL;
    gboolean b = g_key_file_get_boolean(file,"theme",key,&e);
    if (!e)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),b);
}
#define read_color(key,object) read_color_real(SW[(object)],key,f)
#define read_int(key,object) read_int_real(SW[(object)],key,f)
#define read_float(key,object) read_float_real(SW[(object)],key,f)
#define read_bool(key,object) read_bool_real(SW[(object)],key,f)
static void cb_load(GtkWidget *w, gpointer d)
{
    GKeyFile * f;
    gchar * fn, *at, *mt;
    gboolean dist;
    gboolean wasact;
    dist=FALSE;
    GtkTreeIter iter;
    GtkTreeModel *model;
    //first normalize the name
    if (gtk_tree_selection_get_selected(ThemeSelect,&model,&iter))
    {
        gtk_tree_model_get(model,&iter,0,&at,-1);
    }
    else
        return;
    mt = at;
    if (strlen(at)>=1 && at[0] == '*')
    {
        if (strlen(at)>=2)
        {
            mt = at+2;
            dist=TRUE;
        }
        else
        {
            error_dialog("Invalid Theme Name");
            g_free(at);
            return;
        }
    }
    if (strlen(mt)==0)
    {
        g_free(at);
        error_dialog("Invalid Theme Name");
        return;
    }
    if (dist)
        fn = g_strdup_printf(DATA_DIR "/compiz/themes/%s" THEMEENDING,mt);
    else
        fn = g_strdup_printf("%s/.compiz/themes/%s" THEMEENDING,g_get_home_dir(),mt);
    f = g_key_file_new();
    if (!g_key_file_load_from_file(f,fn,G_KEY_FILE_KEEP_COMMENTS,NULL))
    {
        g_free(fn);
        g_key_file_free(f);
        g_free(at);
        error_dialog("Invalid Theme File / Name");
        return;
    }
    gtk_entry_set_text(GTK_ENTRY(EntryBox),mt);
    g_free(at);
    wasact=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(AutoApply),FALSE);
    changed=TRUE;
    read_float("shadow_radius",S_SR);
    read_float("shadow_opacity",S_SA);
    read_int("shadow_offset_x",S_SXO);
    read_int("shadow_offset_y",S_SYO);
    read_float("active_outer_alpha",S_WAOA);
    read_float("inactive_outer_alpha",S_WIOA);
    read_float("active_inner_alpha",S_WAIA);
    read_float("inactive_inner_alpha",S_WIIA);
    read_float("active_button_alpha",S_BAA);
    read_float("inactive_button_alpha",S_BIA);
    read_bool("use_active_colors",S_WAUA);
    read_bool("draw_separator_line",S_WADL);
    read_color("active_outer",S_WAOC);
    read_color("active_inner",S_WAIC);
    read_color("inactive_outer",S_WIOC);
    read_color("inactive_inner",S_WIIC);
    read_color("active_text",S_TAFC);
    read_color("active_text_halo",S_TAOC);
    read_color("inactive_text",S_TIFC);
    read_color("inactive_text_halo",S_TIOC);
    read_color("active_button",S_BAFC);
    read_color("active_button_halo",S_BAOC);
    read_color("inactive_button",S_BIFC);
    read_color("inactive_button_halo",S_BIOC);
    read_color("window_frame_halo",S_WHC);
    read_color("shadow_color",S_SC);
    read_font(f);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(AutoApply),wasact);
    g_key_file_free(f);
    g_free(fn);
}

void write_color_real(GtkWidget * w, char * key, GKeyFile * file)
{
    GdkColor c;
    gchar * ch;
    gtk_color_button_get_color(GTK_COLOR_BUTTON(w),&c);
    ch = g_strdup_printf("#%02x%02x%02x",c.red>>8,c.green>>8,c.blue>>8);
    g_key_file_set_string(file,"theme",key,ch);
    g_free(ch);
}
#define write_color(key,object) write_color_real(SW[(object)],key,f)
#define write_int(key,object) g_key_file_set_integer(f,"theme",key,\
        (int)gtk_range_get_value(GTK_RANGE(SW[(object)])))
void write_float_real(GtkWidget * w, char * key, GKeyFile * file)
{
    gchar ch[G_ASCII_DTOSTR_BUF_SIZE+1];
    g_ascii_dtostr(ch,G_ASCII_DTOSTR_BUF_SIZE,
            gtk_range_get_value(GTK_RANGE(w)));
    g_key_file_set_string(file,"theme",key,ch);
}
#define write_float(key,object) write_float_real(SW[(object)],key,f)
#define write_bool(key,object) g_key_file_set_boolean(f,"theme",key,\
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(SW[(object)])))
static void cb_save(GtkWidget *w, gpointer d)
{
    GKeyFile * f;
    gchar * fn, *at;
    //first normalize the name
    if (! (at=(gchar *) gtk_entry_get_text(GTK_ENTRY(EntryBox))))
        return;
    if (strlen(at)>=1 && at[0] == '*')
    {
        error_dialog("Can't save over system themes\n(or you forgot to enter a name)");
        return;
    }
    if (strlen(at)==0)
    {
        error_dialog("Invalid Theme Name");
        return;
    }
    fn = g_strdup_printf("%s/.compiz/themes/",g_get_home_dir());
    g_mkdir_with_parents(fn,00755);
    g_free(fn);
    fn = g_strdup_printf("%s/.compiz/themes/%s" THEMEENDING,g_get_home_dir(),at);
    f = g_key_file_new();
    g_key_file_load_from_file(f,fn,G_KEY_FILE_KEEP_COMMENTS,NULL);
    write_float("shadow_radius",S_SR);
    write_float("shadow_opacity",S_SA);
    write_int("shadow_offset_x",S_SXO);
    write_int("shadow_offset_y",S_SYO);
    write_float("active_outer_alpha",S_WAOA);
    write_float("inactive_outer_alpha",S_WIOA);
    write_float("active_inner_alpha",S_WAIA);
    write_float("inactive_inner_alpha",S_WIIA);
    write_float("active_button_alpha",S_BAA);
    write_float("inactive_button_alpha",S_BIA);
    write_bool("use_active_colors",S_WAUA);
    write_bool("draw_separator_line",S_WADL);
    write_color("active_outer",S_WAOC);
    write_color("active_inner",S_WAIC);
    write_color("inactive_outer",S_WIOC);
    write_color("inactive_inner",S_WIIC);
    write_color("active_text",S_TAFC);
    write_color("active_text_halo",S_TAOC);
    write_color("inactive_text",S_TIFC);
    write_color("inactive_text_halo",S_TIOC);
    write_color("active_button",S_BAFC);
    write_color("active_button_halo",S_BAOC);
    write_color("inactive_button",S_BIFC);
    write_color("inactive_button_halo",S_BIOC);
    write_color("window_frame_halo",S_WHC);
    write_color("shadow_color",S_SC);
    write_font(f);
    if (g_file_test(fn,G_FILE_TEST_EXISTS))
    {
        if (!confirm_dialog("Overwrite Theme %s?",at))
        {
            return;
        }
    }
    at = g_key_file_to_data(f,NULL,NULL);
    if (at && !g_file_set_contents(fn,at,-1,NULL))
    {
        error_dialog("Couldn't Write Theme");
        g_free(at);
    }
    else if (!at)
    {
        error_dialog("Couldn't Form Theme");
    }
    else
    {
        info_dialog("Theme Saved");
        g_free(at);
    }
    g_key_file_free(f);
    g_free(fn);
    refresh_theme_list();
}
static void cb_delete(GtkWidget *w, gpointer d)
{
    GtkTreeIter iter;
    GtkTreeModel *model;
    gchar * fn, *at;
    //first normalize the name
    if (gtk_tree_selection_get_selected(ThemeSelect,&model,&iter))
    {
        gtk_tree_model_get(model,&iter,0,&at,-1);
    }
    else
        return;

    if (strlen(at)>=1 && at[0] == '*')
    {
        g_free(at);
        error_dialog("Can't delete system themes\n(or you forgot to enter a name)");
        return;
    }
    if (strlen(at)==0)
    {
        g_free(at);
        error_dialog("Invalid Theme Name");
        return;
    }
    fn = g_strdup_printf("%s/.compiz/themes/%s" THEMEENDING,g_get_home_dir(),at);
    if (g_file_test(fn,G_FILE_TEST_EXISTS))
    {
        if (!confirm_dialog("Are you sure you want to delete %s?",at))
        {
            g_free(at);
            g_free(fn);
            return;
        }
        if (g_unlink(fn))
        {
            error_dialog("Couldn't delete theme");
        }
        else
        {
            info_dialog("Theme deleted");
        }
        refresh_theme_list();
    }
    else
    {
        error_dialog("No such theme to delete");
    }
    g_free(at);
    g_free(fn);
}
static void cb_revert(GtkWidget *w, gpointer d)
{
    changed=FALSE;
    init_settings();
}
static void cb_apply_int(GtkWidget *w, gpointer d)
{
    gchar * k = (gchar *) d;
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply)))
    {
        gconf_client_set_int(gconf,k,
                (int)gtk_range_get_value(GTK_RANGE(w)),NULL);
    }
    else
        changed=TRUE;
}
static void cb_apply_float(GtkWidget *w, gpointer d)
{
    gchar * k = (gchar *) d;
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply)))
    {
        gconf_client_set_float(gconf,k,
                gtk_range_get_value(GTK_RANGE(w)),NULL);
    }
    else
        changed=TRUE;
}
static void cb_apply_bool(GtkWidget *w, gpointer d)
{
    gchar * k = (gchar *) d;
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply)))
    {
        gconf_client_set_bool(gconf,k,
                gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)),NULL);
    }
    else
        changed=TRUE;
}
static void cb_apply_color(GtkWidget *w, gpointer d)
{
    gchar * k = (gchar *) d;
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(AutoApply)))
    {
        set_color_real(w,k);
    }
    else
        changed=TRUE;
}
static void cb_apply(GtkWidget *w, gpointer d)
{
    apply_settings();
    changed=FALSE;
}
static void cb_combo(GtkWidget *w,gpointer d)
{
    cb_load(w,d);
}
static void cb_sth(GtkWidget * w, gpointer d)
{
    set_bool("use_new_decor",S_TH);
}
static void cb_main_destroy (GtkWidget * w, gpointer d)
{
    gtk_main_quit();
}
static void cb_auto (GtkWidget * w, gpointer d)
{
    set_bool("auto_apply",S_AAPLY);
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) && changed)
        cb_apply(w,d);
}
#define gtk_box_pack_startC(a,b,c,d,e) gtk_box_pack_start(GTK_BOX(a),b,c,d,e)
#define gtk_box_pack_endC(a,b,c,d,e) gtk_box_pack_end(GTK_BOX(a),b,c,d,e)
#define gtk_container_addC(a,b) gtk_container_add(GTK_CONTAINER(a),b)
#define gtk_container_set_border_widthC(a,b) gtk_container_set_border_width(GTK_CONTAINER(a),b)
int main (int argc, char * argv[])
{
    GtkWidget * w;
    
    gtk_init (&argc, &argv);
    
    mainWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    gtk_window_set_title (GTK_WINDOW (mainWindow), "Compiz Themer " VERSION);

    //gtk_window_set_default_size (GTK_WINDOW (mainWindow), 0, 0);
    gtk_window_set_resizable (GTK_WINDOW (mainWindow), FALSE);

    g_signal_connect(G_OBJECT (mainWindow), "destroy", G_CALLBACK(cb_main_destroy), NULL);
    
    gtk_window_set_default_icon_from_file (PIXMAPS_DIR "/gcompizthemer-icon.png", NULL);

    gtk_container_set_border_widthC (GTK_CONTAINER (mainWindow), 5);

    //here's where we begin our layout.
    //
    //

    w = gtk_vbox_new(FALSE,2);
    gtk_container_addC(GTK_CONTAINER (mainWindow), w);
    {
        GtkWidget * w2;
        
        w2 = gtk_check_button_new_with_label("Enable Customized Decorator");
        gtk_box_pack_startC(w,w2,FALSE,FALSE,0);
        SW[S_TH] = w2;
        g_signal_connect(w2,"toggled",G_CALLBACK(cb_sth),NULL);

        w2 = gtk_frame_new("Theme");
        gtk_box_pack_startC(w,w2,FALSE,FALSE,0);
        {
            GtkWidget * w3;

            w3 = gtk_vbox_new(FALSE,2);
            gtk_container_addC(w2,w3);
            gtk_container_set_border_widthC(w3,8);
            {
                GtkWidget * w4;

                ThemeList = gtk_list_store_new(1,G_TYPE_STRING);
                ThemeRenderer = gtk_cell_renderer_text_new();
                ThemeColumn = gtk_tree_view_column_new_with_attributes
                    ("Name",ThemeRenderer,"text",0,NULL);
                w4 = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ThemeList));
                gtk_tree_view_append_column(GTK_TREE_VIEW(w4),ThemeColumn);
                ThemeSelector = w4;
                ThemeSelect = gtk_tree_view_get_selection(
                        GTK_TREE_VIEW(ThemeSelector));
                gtk_tree_selection_set_mode(ThemeSelect,GTK_SELECTION_SINGLE);
                g_signal_connect(ThemeSelect,"changed",
                        G_CALLBACK(cb_combo),NULL);
                w4 = gtk_scrolled_window_new(NULL,NULL);
                gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w4),
                        GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
                gtk_container_addC(w4,ThemeSelector);
                gtk_box_pack_startC(w3,w4,TRUE,TRUE,0);
                w4 = gtk_hbox_new(TRUE,2);
                gtk_box_pack_startC(w3,w4,FALSE,FALSE,0);
                {
                    GtkWidget *w5;

                    /*w5 = gtk_button_new_from_stock(GTK_STOCK_OPEN);
                    gtk_box_pack_startC(w4,w5,TRUE,TRUE,0);
                    g_signal_connect(w5,"clicked",G_CALLBACK(cb_load),NULL);*/
                    //open button was rather useless
                    //
                    
                    /*w5 = gtk_button_new_from_stock(GTK_STOCK_SAVE_AS);
                    gtk_box_pack_startC(w4,w5,TRUE,TRUE,0);

                    w5 = gtk_button_new_from_stock(GTK_STOCK_OPEN);
                    gtk_box_pack_startC(w4,w5,TRUE,TRUE,0);*/
                    //TODO - make these work.

                    w5 = gtk_button_new_from_stock(GTK_STOCK_DELETE);
                    gtk_box_pack_startC(w4,w5,TRUE,TRUE,0);
                    g_signal_connect(w5,"clicked",G_CALLBACK(cb_delete),NULL);

                    w5 = gtk_button_new_from_stock(GTK_STOCK_APPLY);
                    gtk_box_pack_startC(w4,w5,TRUE,TRUE,0);
                    g_signal_connect(w5,"clicked",G_CALLBACK(cb_apply),NULL);
                }

            }
        }

        w2 = gtk_check_button_new_with_label("Auto-Apply Changes");
        gtk_box_pack_startC(w,w2,FALSE,FALSE,0);
        AutoApply = w2;
        SW[S_AAPLY]=w2;
        g_signal_connect(w2,"toggled",G_CALLBACK(cb_auto),NULL);
        
        Cheating = gtk_expander_new("Edit");
        gtk_box_pack_startC(w,Cheating,FALSE,FALSE,0);
        gtk_expander_set_expanded(GTK_EXPANDER(Cheating),FALSE);
        
        Cheating2 = gtk_vbox_new(FALSE,2);
        gtk_container_addC(Cheating,Cheating2);

        w2 = gtk_hbox_new(FALSE,2);
        gtk_box_pack_startC(Cheating2,w2,FALSE,FALSE,0);
        {
            GtkWidget *w3;
            w3 = gtk_label_new("Name:");
            gtk_box_pack_startC(w2,w3,FALSE,FALSE,0);

            w3 = gtk_entry_new();
            gtk_box_pack_startC(w2,w3,TRUE,TRUE,0);
            EntryBox = w3;
        }

        w2 = gtk_notebook_new();
        gtk_box_pack_startC(Cheating2,w2,FALSE,FALSE,0);
        {
            GtkWidget * w3;
            w3 = gtk_vbox_new(FALSE,2);
            gtk_container_set_border_widthC(w3,8);
            gtk_notebook_append_page(GTK_NOTEBOOK(w2),w3,gtk_label_new("Active Window"));
            {
                GtkWidget * w4;
                GtkWidget * cch;
                GtkWidget * cch2;
                cch = gtk_hbox_new(TRUE,2);
                gtk_box_pack_startC(w3,cch,FALSE,FALSE,0);

                w4 = gtk_frame_new("Borders");
                gtk_box_pack_startC(cch,w4,TRUE,TRUE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outer Blend Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WAOC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "active_outer");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outer Blend Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,1.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WAOA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "active_outer_alpha");
                        }
                        w6 = gtk_hseparator_new();
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Inner Blend Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WAIC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "active_inner");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Inner Blend Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,1.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WAIA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "active_inner_alpha");
                        }

                        w6 = gtk_hseparator_new();
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);

                        w6 = gtk_check_button_new_with_label("Use these colors for whole active frame");
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        SW[S_WAUA]=w6;
                        g_signal_connect(w6,"toggled",
                                G_CALLBACK(cb_apply_bool),
                                CDR "use_active_colors");

                        w6 = gtk_check_button_new_with_label("Draw line between titlebar and window/frame");
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        SW[S_WADL]=w6;
                        g_signal_connect(w6,"toggled",
                                G_CALLBACK(cb_apply_bool),
                                CDR "draw_separator_line");
                    }
                }
                cch2 = gtk_vbox_new(FALSE,2);
                gtk_box_pack_startC(cch,cch2,TRUE,TRUE,0);
                
                w4 = gtk_frame_new("Title Text");
                gtk_box_pack_startC(cch2,w4,FALSE,FALSE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;
                        
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Fill Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_TAFC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "active_text");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outline Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_TAOC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "active_text_halo");
                        }
                    }
                }
                w4 = gtk_frame_new("Buttons");
                gtk_box_pack_startC(cch2,w4,FALSE,FALSE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;

                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Fill Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_BAFC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "active_button");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outline Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_BAOC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "active_button_halo");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,1.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_BAA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "active_button_alpha");
                        }
                    }
                }

            }

            w3 = gtk_vbox_new(FALSE,2);
            gtk_container_set_border_widthC(w3,8);
            gtk_notebook_append_page(GTK_NOTEBOOK(w2),w3,gtk_label_new("Inactive Window"));
            {
                GtkWidget * w4;
                GtkWidget * cch;
                GtkWidget * cch2;
                
                cch = gtk_hbox_new(TRUE,2);
                gtk_box_pack_startC(w3,cch,FALSE,FALSE,0);
                
                w4 = gtk_frame_new("Borders");
                gtk_box_pack_startC(cch,w4,TRUE,TRUE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outer Blend Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WIOC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "inactive_outer");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outer Blend Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,1.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WIOA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "inactive_outer_alpha");
                        }
                        w6 = gtk_hseparator_new();
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);

                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Inner Blend Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WIIC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "inactive_inner");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Inner Blend Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,1.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WIIA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "inactive_inner_alpha");
                        }
                    }
                }
                
                cch2 = gtk_vbox_new(FALSE,2);
                gtk_box_pack_startC(cch,cch2,TRUE,TRUE,0);
                
                w4 = gtk_frame_new("Title Text");
                gtk_box_pack_startC(cch2,w4,FALSE,FALSE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;
                        
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Fill Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_TIFC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "inactive_text");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outline Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_TIOC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "inactive_text_halo");
                        }
                    }
                }
                w4 = gtk_frame_new("Buttons");
                gtk_box_pack_startC(cch2,w4,FALSE,FALSE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;

                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Fill Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_BIFC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "inactive_button");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Outline Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_BIOC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "inactive_button_halo");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,1.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_BIA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "inactive_button_alpha");
                        }
                    }
                }

            }
            w3 = gtk_vbox_new(FALSE,2);
            gtk_container_set_border_widthC(w3,8);
            gtk_notebook_append_page(GTK_NOTEBOOK(w2),w3,gtk_label_new("Global"));
            {
                GtkWidget * w4;
                GtkWidget * cch;
                GtkWidget * cch2;

                cch = gtk_hbox_new(TRUE,2);
                gtk_box_pack_startC(w3,cch,TRUE,TRUE,0);
                
                w4 = gtk_frame_new("Shadows");
                gtk_box_pack_startC(cch,w4,TRUE,TRUE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;

                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_SC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "shadow_color");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Opacity");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.01,6.0,0.01);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_SA]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "shadow_opacity");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Radius");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(0.0,48.0,0.1);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_SR]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_float),
                                    CDR "shadow_radius");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("X Offset");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(-16.0,16.0,1);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_SXO]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_int),
                                    CDR "shadow_offset_x");
                        }
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;

                            w7 = gtk_label_new("Y Offset");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_hscale_new_with_range(-16,16,1);
                            gtk_widget_set_size_request(w7,200,-1);
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_SYO]=w7;
                            g_signal_connect(w7,"value-changed",
                                    G_CALLBACK(cb_apply_int),
                                    CDR "shadow_offset_y");
                        }
                    }
                }

                cch2 = gtk_vbox_new(FALSE,2);
                gtk_box_pack_startC(cch,cch2,TRUE,TRUE,0);
                
                w4 = gtk_frame_new("Windows");
                gtk_box_pack_startC(cch2,w4,TRUE,TRUE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;
                        
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;
                            w7 = gtk_label_new("Outline Color");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_color_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            SW[S_WHC]=w7;
                            g_signal_connect(w7,"color-set",
                                    G_CALLBACK(cb_apply_color),
                                    CDR "window_frame_halo");
                        }
                    }
                }
                w4 = gtk_frame_new("Title Text");
                gtk_box_pack_startC(cch2,w4,TRUE,TRUE,0);
                {
                    GtkWidget * w5;
                    w5 = gtk_vbox_new(FALSE, 2);
                    gtk_container_set_border_widthC(w5,8);
                    gtk_container_addC(w4,w5);
                    {
                        GtkWidget * w6;
                        
                        w6 = gtk_hbox_new(FALSE,2);
                        gtk_box_pack_startC(w5,w6,FALSE,FALSE,0);
                        {
                            GtkWidget * w7;
                            w7 = gtk_label_new("Font");
                            gtk_box_pack_startC(w6,w7,FALSE,FALSE,0);

                            w7 = gtk_font_button_new();
                            gtk_box_pack_endC(w6,w7,FALSE,FALSE,0);
                            FontButton = w7;
                            g_signal_connect(w7,"font-set",
                                    G_CALLBACK(cb_set_font),
                                    NULL);
                        }
                    }
                }
            }
        }

        w2 = gtk_hbox_new(TRUE,2);
        gtk_box_pack_startC(Cheating2,w2,FALSE,FALSE,0);
        {
            GtkWidget *w3;
            w3 = gtk_button_new_from_stock(GTK_STOCK_SAVE);
            gtk_box_pack_startC(w2,w3,TRUE,TRUE,0);
            g_signal_connect(w3,"clicked",G_CALLBACK(cb_save),NULL);

            w3 = gtk_button_new_from_stock(GTK_STOCK_REVERT_TO_SAVED);
            gtk_box_pack_startC(w2,w3,TRUE,TRUE,0);
            g_signal_connect(w3,"clicked",G_CALLBACK(cb_revert),NULL);
        }
    }

    gtk_widget_show_all(mainWindow);

    init_settings();
    get_bool("auto_apply",S_AAPLY);
    refresh_theme_list();

    gtk_main();
    return 0;
}
